extern crate nom;

use renderer::compile_template;
use renderer::CompiledTemplate;
use renderer::DustRenderer;
use std::env;
use std::fs;
use std::io::{self, Read};
use std::path::Path;

mod parser;
mod renderer;

fn main() {
    let context = read_context_from_stdin();

    let argv: Vec<String> = env::args().collect();
    if argv.len() < 2 {
        panic!("Need to pass templates");
    }
    let template_paths = &argv[1..];
    let template_contents: Vec<(String, String)> = template_paths
        .iter()
        .map(|p| {
            let template_content = fs::read_to_string(&p).unwrap();
            (p.to_string(), template_content)
        })
        .collect();
    let compiled_templates: Vec<CompiledTemplate> = template_contents
        .iter()
        .map(|(p, contents)| template_from_file(p, contents))
        .collect();
    let mut dust_renderer = DustRenderer::new();
    compiled_templates.iter().for_each(|template| {
        dust_renderer.load_source(template);
    });
    let main_template_name = &compiled_templates
        .first()
        .expect("There should be more than 1 template")
        .name;
    println!(
        "{}",
        dust_renderer
            .render(main_template_name, context)
            .expect("Failed to render")
    );
}

fn template_from_file<'a>(file_path: &str, file_contents: &'a str) -> CompiledTemplate<'a> {
    let path: &Path = Path::new(file_path);
    let name = path.file_stem().unwrap();
    compile_template(file_contents, name.to_string_lossy().to_string())
        .expect("Failed to compile template")
}

fn read_context_from_stdin() -> serde_json::map::Map<String, serde_json::Value> {
    let mut buffer = String::new();
    io::stdin()
        .read_to_string(&mut buffer)
        .expect("Failed to read stdin");

    let parsed: serde_json::Value = serde_json::from_str(&buffer).expect("Failed to parse json");
    match parsed {
        serde_json::Value::Object(obj) => obj,
        _ => panic!("Expected context to be an object"),
    }
}
